/*
    Copyright (C) 2000 Paul Barton-Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: audiothing.h,v 1.7 2000/11/04 04:53:33 pbd Exp $
*/

#ifndef __audiothing_h__
#define __audiothing_h__

#include <sigc++/signal_system.h>
#include <string>

/* an abstract base class for objects that can be connected
   to a "pull" audio device. The audio device object will
   call "process" every so often when there is data for
   the AudioThing to process, and/or data is needed by the
   audio device. These two things are assumed to happen
   synchronously with each other.
*/

class AudioThing : public SigC::Object {

  public:
	AudioThing (const string &str) {
		_name = str;
	}
	virtual ~AudioThing() {};

	virtual int process (size_t nsamples, size_t nbytes) = 0;
	virtual int post_process () = 0;
	virtual unsigned int current_frame () = 0;
	virtual size_t nchannels () = 0;

	/* this is called after a device has
	   connected to this AudioThing.
	*/

	virtual void post_connect() = 0; 

	/* this is called when an xrun occurs. if it returns true,
	   the MCD will try to recover from the xrun. otherwise
	   it will stop.
	*/

	virtual bool xrun_recoverable (ssize_t xrun_frames) = 0;

	SigC::Signal1<void,AudioThing *> going_away;

	const string &name() { return _name; }

  protected:
	string _name;
};


#endif /* __audiothing_h__ */
