/* gtkeditablewavebuffer.h: Interface for editable wavebuffer.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __GTK_EDITABLE_WAVE_BUFFER_H__
#define __GTK_EDITABLE_WAVE_BUFFER_H__

#include "grange.h"
#include "gtkwavebuffer.h"

#include "../../../seq.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define GTK_TYPE_EDITABLE_WAVE_BUFFER            (gtk_editable_wave_buffer_get_type ())
#define GTK_EDITABLE_WAVE_BUFFER(obj)            (GTK_CHECK_CAST ((obj), GTK_TYPE_EDITABLE_WAVE_BUFFER, GtkEditableWaveBuffer))
#define GTK_EDITABLE_WAVE_BUFFER_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_EDITABLE_WAVE_BUFFER, GtkEditableWaveBufferClass))
#define GTK_IS_EDITABLE_WAVE_BUFFER(obj)         (GTK_CHECK_TYPE ((obj), GTK_TYPE_EDITABLE_WAVE_BUFFER))
#define GTK_IS_EDITABLE_WAVE_BUFFER_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_EDITABLE_WAVE_BUFFER))


typedef struct _GtkEditableWaveBuffer         GtkEditableWaveBuffer;
typedef struct _GtkEditableWaveBufferClass    GtkEditableWaveBufferClass;

struct _GtkEditableWaveBuffer
{
  GtkWaveBuffer object;

  gboolean  modified;
  GRange    modified_range;
  guint     modified_idle;
};

struct _GtkEditableWaveBufferClass
{
  GtkWaveBufferClass object_class;

  /* Signals */
  void (*modified)      (GtkWidget *widget, 
			 guint32 channel_mask,
                         GRange *range);
  void (*insert_data)   (GtkWidget *widget, 
			 guint32 channel_mask,
                         GRange *range);
  void (*delete_data)   (GtkWidget *widget, 
			 guint32 channel_mask,
                         GRange *range);

  /* Pure abstract functions. */
  gint (*insert)        (GtkEditableWaveBuffer *editable_wave_buffer,
                         guint32 start,
			 piece_t *piece);
  gint (*insert_sequence)(GtkEditableWaveBuffer *editable_wave_buffer,
			  guint32 start,
			  sequence_t *);
  gint (*replace)       (GtkEditableWaveBuffer *editable_wave_buffer,
                         guint32 start,
			 piece_t *piece);
  gint (*replace_sequence)(GtkEditableWaveBuffer *editable_wave_buffer,
			   guint32 start,
			   sequence_t *seq);
  /* should be called "delete", but C++ won't stand for that */
  gint (*remove)        (GtkEditableWaveBuffer *editable_wave_buffer,
                         guint32 start,
                         guint32 length);
  gint (*undo)          (GtkEditableWaveBuffer *editable_wave_buffer,    
			 guint32 undo_depth);
  gint (*redo)          (GtkEditableWaveBuffer *editable_wave_buffer,    
			 guint32 undo_depth);
};

extern guint   modified_signal;
extern guint   insert_data_signal;
extern guint   delete_data_signal;

GtkType        gtk_editable_wave_buffer_get_type       (void);
void           gtk_editable_wave_buffer_set_modified   (GtkEditableWaveBuffer *editable,
							 guint32 start,
							 guint32 length);
gint           gtk_editable_wave_buffer_insert         (GtkEditableWaveBuffer *editable_wave_buffer,
							 guint32 pos,
							 piece_t *piece);
gint           gtk_editable_wave_buffer_insert_sequence (GtkEditableWaveBuffer *editable_wave_buffer,
							 guint32 pos,
							 sequence_t *seq);
gint           gtk_editable_wave_buffer_replace        (GtkEditableWaveBuffer *editable_wave_buffer,
							 guint32 pos,
							 piece_t *piece);
gint           gtk_editable_wave_buffer_delete         (GtkEditableWaveBuffer *editable_wave_buffer,
							 guint32 start,
							 guint32 length);

gint           gtk_editable_wave_buffer_undo           (GtkEditableWaveBuffer *editable_wave_buffer,
							guint32 undo_depth);
gint           gtk_editable_wave_buffer_redo           (GtkEditableWaveBuffer *editable_wave_buffer,
							guint32 undo_depth);
#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __GTK_EDITABLE_WAVE_BUFFER_H__ */

