;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  mididev.scm
;;;  midi-device config variable definition
;;;
;;;  Bill Gribble <grib@billgribble.com> 2/21/2000
;;;  $Id: mididev.scm,v 1.3 2000/10/04 16:45:31 pbd Exp $
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(depend 'simple-obj)
(depend 'enumeration)
(support 'mididev)

;;  the "mididev" class.  MIDI device definitions are similar to 
;;  soundcard definitions, but with some slightly different enums.

(define <mididev> 
  (make-simple-class 'mididev '(device tag type mode)))

(define (mididev? self)
  (eq? (simple-obj-type self) 'mididev))

(define (mididev:get-device self)
  (simple-obj-getter self <mididev> 'device))

(define (mididev:set-device! self value)
  (if (string? value)
      (simple-obj-setter self <mididev> 'device value)
      (error "mididev:set-device!: device must be a string.")))

(define (mididev:get-tag self)
  (simple-obj-getter self <mididev> 'tag))

(define (mididev:set-tag! self value)
  (if (string? value)
      (simple-obj-setter self <mididev> 'tag value)
      (error "mididev:set-device!: tag must be a string.")))

(define (mididev:get-type self)
  (simple-obj-getter self <mididev> 'type))

(define (mididev:set-type! self value)
  (if (string? value)
      (simple-obj-setter self <mididev> 'type value)
      (error "mididev:set-device!: type must be a string.")))

(define (mididev:get-mode self)
  (simple-obj-getter self <mididev> 'mode))

(define (mididev:set-mode! self value)
  (if (string? value)
      (simple-obj-setter self <mididev> 'mode value)
      (error "mididev:set-device!: mode must be a string.")))

(define (make-mididev . args)
  (let* ((kv-pairs    (map (lambda (elt) (cons (car elt) (cadr elt)))
                           (if (pair? args) (car args) '())))
         (device      (assq 'device: kv-pairs))
	 (tag         (assq 'tag: kv-pairs))
         (type        (assq 'type: kv-pairs))
         (mode        (assq 'mode: kv-pairs)))
    
    ;; the only sanity checking on device is that it's a string.
    (if (pair? device)
        (if (string? (cdr device))
            (set! device (cdr device))
            (error "make-mididev: device spec must be a string."))
        (set! device "/dev/null"))

    ;; the only sanity checking on device is that it's a string.
    (if (pair? tag)
        (if (string? (cdr tag))
            (set! tag (cdr tag))
            (error "make-mididev: tag must be a string."))
        (set! tag "foobar"))

    ;; the only sanity checking on device is that it's a string.
    (if (pair? type)
        (if (string? (cdr type))
            (set! type (cdr type))
            (error "make-mididev: type must be a string."))
        (set! type "ALSA/RAW"))
    
    ;; mode must be a symbol out of %midi-mode-enum
    (if (pair? mode)
        (if (string? (cdr mode))
            (set! mode (cdr mode))
            (error "make-mididev: mode must be a string."))
        (set! mode "output"))

    (let ((self (make-simple-obj <mididev>)))
      (mididev:set-device! self device)
      (mididev:set-tag! self tag)
      (mididev:set-type! self type)
      (mididev:set-mode! self mode)
      self)))
