# Configure paths for Libmidi++
# Based on libsigc++ script by Karl Nelson

dnl Test for Libmidi++, and define MIDIPP_CFLAGS and MIDIPP_LIBS
dnl   to be used as follows:
dnl AM_PATH_MIDIPP(MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]])
dnl
AC_DEFUN(AM_PATH_MIDIPP,
[dnl 
dnl Get the cflags and libraries from the midipp-config script
dnl

dnl
dnl Prefix options
dnl
AC_ARG_WITH(midipp-prefix,
[  --with-midipp-prefix=PREFIX
                          Prefix where libmidi++ is installed (optional)]
, midipp_config_prefix="$withval", midipp_config_prefix="")

AC_ARG_WITH(midipp-exec-prefix,
[  --with-midipp-exec-prefix=PREFIX 
                          Exec prefix where libmidi++ is installed (optional)]
, midipp_config_exec_prefix="$withval", midipp_config_exec_prefix="")

AC_ARG_ENABLE(midipptest, 
[  --disable-midipptest     Do not try to compile and run a test libmidi++ 
                          program],
, enable_midipptest=yes)

dnl
dnl Prefix handling
dnl
  if test x$midipp_config_exec_prefix != x ; then
     midipp_config_args="$midipp_config_args --exec-prefix=$midipp_config_exec_prefix"
     if test x${MIDIPP_CONFIG+set} != xset ; then
        MIDIPP_CONFIG=$midipp_config_exec_prefix/bin/midi++-config
     fi
  fi
  if test x$midipp_config_prefix != x ; then
     midipp_config_args="$midipp_config_args --prefix=$midipp_config_prefix"
     if test x${MIDIPP_CONFIG+set} != xset ; then
        MIDIPP_CONFIG=$midipp_config_prefix/bin/midi++-config
     fi
  fi

dnl
dnl See if midipp-config is alive
dnl
  AC_PATH_PROG(MIDIPP_CONFIG, midi++-config, no)
  midipp_version_min=$1

dnl
dnl  Version check
dnl
  AC_MSG_CHECKING(for libmidipp - version >= $midipp_version_min)
  no_midipp=""
  if test "$MIDIPP_CONFIG" = "no" ; then
    no_midipp=yes
  else
    midipp_version=`$MIDIPP_CONFIG --version`

    MIDIPP_CFLAGS=`$MIDIPP_CONFIG $midipp_config_args --cflags`
    MIDIPP_LIBS=`$MIDIPP_CONFIG $midipp_config_args --libs`

    midipp_major_version=`echo $midipp_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    midipp_minor_version=`echo $midipp_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    midipp_micro_version=`echo $midipp_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`

    midipp_major_min=`echo $midipp_major_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    midipp_minor_min=`echo $midipp_minor_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    midipp_micro_min=`echo $midipp_micro_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`

    midipp_version_proper=`expr \
        $midipp_major_version \> $midipp_major_min \| \
        $midipp_major_version \= $midipp_major_min \& \
        $midipp_minor_version \> $midipp_minor_min \| \
        $midipp_major_version \= $midipp_major_min \& \
        $midipp_minor_version \= $midipp_minor_min \& \
        $midipp_micro_version \>= $midipp_micro_min `

    if test "$midipp_version_proper" = "1" ; then
      AC_MSG_RESULT([$midipp_major_version.$midipp_minor_version.$midipp_micro_version])
    else
      AC_MSG_RESULT(no)
      no_midipp=yes
    fi

    if test "X$no_midipp" = "Xyes" ; then
      enable_midipptest=no
    fi

    AC_LANG_SAVE
    AC_LANG_CPLUSPLUS

dnl
dnl
dnl
    if test "x$enable_midipptest" = "xyes" ; then
      AC_MSG_CHECKING(if libmidipp sane)
      ac_save_CXXFLAGS="$CXXFLAGS"
      ac_save_LIBS="$LIBS"
      CXXFLAGS="$CXXFLAGS $MIDIPP_CFLAGS"
      LIBS="$LIBS $MIDIPP_LIBS"

      rm -f conf.midipptest
      AC_TRY_RUN([
#include <stdio.h>
#include <midi++/version.h>
#include <pbd/transmitter.h>

Transmitter error(Transmitter::Error);
Transmitter info(Transmitter::Info);
Transmitter warning(Transmitter::Warning);
Transmitter fatal(Transmitter::Fatal);

int main(int argc,char **argv)
  {
   if (midipp_major_version!=$midipp_major_version ||
       midipp_minor_version!=$midipp_minor_version ||
       midipp_micro_version!=$midipp_micro_version)
     { printf("(%d.%d.%d) ",
         midipp_major_version,midipp_minor_version,midipp_micro_version);
       return 1;
     }
  }

],[
  AC_MSG_RESULT(yes)
],[
  AC_MSG_RESULT(no)
  no_midipp=yes
]
,[echo $ac_n "cross compiling; assumed OK... $ac_c"])

       CXXFLAGS="$ac_save_CXXFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi

  dnl
  dnl
  if test "x$no_midipp" = x ; then
     ifelse([$2], , :, [$2])     
  else
     MIDIPP_CFLAGS=""
     MIDIPP_LIBS=""
     ifelse([$3], , :, [$3])
  fi

  AC_LANG_RESTORE

  AC_SUBST(MIDIPP_CFLAGS)
  AC_SUBST(MIDIPP_LIBS)
])

