/*
    Copyright (C) 2000 Paul Barton-Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: location_ui.h,v 1.14 2000/10/28 13:27:38 pbd Exp $
*/

#ifndef __ardour_location_ui_h__
#define __ardour_location_ui_h__

#include <vector>
#include <gtk--.h>

#include <pbd/touchable.h>

#include <gtkmmext/pixmap_button.h>
#include <gtkmmext/click_box.h>

#include "location.h"

namespace Gtkmmext {
	class LED;
};

namespace ALSA {
	class MultiChannelDevice;
}

namespace ARDOUR {
	class Tape;
}

class AudioThing;

class Location_UI : public Gtk::Fixed, public Touchable {
  public:
	Location_UI (ALSA::MultiChannelDevice *, 
		     Gtk::Widget &pixmap_window);

	void connect_to_tape (ARDOUR::Tape *t);
	void touch ();

  private:
	ALSA::MultiChannelDevice *mcd;
	ARDOUR::Tape *tape;

	Gtk::Adjustment start_adjustment;
	Gtkmmext::AutoSpin start_spinner;

	Gtk::Adjustment end_adjustment;
	Gtkmmext::AutoSpin end_spinner;
	
	Gtkmmext::PixmapButton   start_grab_tape;
	Gtkmmext::PixmapButton   start_forward;
	Gtkmmext::PixmapButton   start_backward;
	Gtk::Label start_forward_label;
	Gtk::Label start_grab_label;
	Gtk::Label start_backward_label;
	Gtk::VBox start_forward_box;
	Gtk::VBox start_grab_box;
	Gtk::VBox start_backward_box;
	Gtk::HBox start_sub_hbox;

	Gtkmmext::PixmapButton   end_grab_tape;
	Gtkmmext::PixmapButton   end_forward;
	Gtkmmext::PixmapButton   end_backward;
	Gtk::Label end_forward_label;
	Gtk::Label end_grab_label;
	Gtk::Label end_backward_label;
	Gtk::VBox end_forward_box;
	Gtk::VBox end_grab_box;
	Gtk::VBox end_backward_box;
	Gtk::HBox end_sub_hbox;

	Gtk::HBox  start_end_hbox;

	Gtkmmext::PixmapButton   next_location;
	Gtkmmext::PixmapButton   previous_location;
	Gtk::Label next_label;
	Gtk::Label previous_label;
	Gtk::HBox next_box;
	Gtk::HBox previous_box;
	Gtk::HBox next_prev_hbox;

	Gtk::VBox  left_vpacker;
	Gtk::VBox  middle_vpacker;
	Gtk::VBox  right_vpacker;
	Gtk::HBox  hpacker;
	
	Gtk::Entry  name_entry;
	Gtk::Label  name_label;
	Gtk::HBox   name_hbox;
	Gtk::VBox   name_vbox;
	Gtk::Frame  name_frame;
	
	Gtk::Entry start_display;
	Gtk::Label start_label;
	Gtk::Frame start_frame;
	Gtk::VBox  start_vbox;

	Gtk::Entry end_display;
	Gtk::Label end_label;
	Gtk::Frame end_frame;
	Gtk::VBox  end_vbox;

	Gtk::Pixmap             *start_pixmap;
	Gtk::EventBox            start_button;
	Gtk::VBox                start_button_vbox;

	Gtk::Pixmap             *end_pixmap;
	Gtk::EventBox            end_button;
	Gtk::VBox                end_button_vbox;

	Gtk::Pixmap             *set_next_pixmap;
	Gtk::EventBox            set_next_button;
	Gtk::VBox                set_next_button_vbox;

	Gtkmmext::PixmapButton  *auto_punch_indicator;
	Gtk::Label               auto_punch_label;
	Gtk::HBox                auto_punch_box;

	Gtkmmext::PixmapButton  *auto_loop_indicator;
	Gtk::Label               auto_loop_label;
	Gtk::HBox                auto_loop_box;

	Gtk::VBox                auto_box;

	Location *location;
	
	SigC::Connection name_connection;
	SigC::Connection start_connection;
	SigC::Connection end_connection;

	bool yes_its_me;

	gint fake_button_press (GdkEventButton *, int);

	void location_change (Location *);
	void map_location ();

	void map_start (Location*);
	void map_end (Location*);
	void map_name (Location*);

	/* its not clear if these will ever be hooked up
	   change signals from the location, but we'll
	   leave the prototypes as if they were for now.
	*/

	void map_auto_punch (Location*);
	void map_auto_loop (Location*);

	void up ();
	void down ();

	void set_location_name ();
	void set_start ();
	void set_end ();
	void set_start_from_tape ();
	void set_end_from_tape   ();
	gint set_next_from_tape (GdkEventButton *);

	void auto_punch_toggle ();
	void auto_loop_toggle ();

	gint locate_start (GdkEventButton *);
	gint locate_end (GdkEventButton *);

	void disconnect_from_tape (AudioThing *);
};

#endif /* __ardour_location_ui_h__ */
