/*
    Copyright (C) 2000 Paul Barton-Davis
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: main.cc,v 1.20 2000/10/31 03:51:38 pbd Exp $
*/

#include <string>

#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <getopt.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/errno.h>
#include <guile/gh.h>

#include <midi++/port.h>
#include <midi++/port_request.h>
#include <midi++/manager.h>
#include <midi++/mmc.h>

#include <pbd/error.h>
#include <pbd/rt.h>
#include <pbd/textreceiver.h>
#include <pbd/pthread_spinlock.h>

#include "version.h"
#include "ardour_ui.h"
#include "alsa_device.h"
#include "configuration.h"
#include "config.h"

/* our only global variables */

Transmitter  error (Transmitter::Error);
Transmitter  info (Transmitter::Info);
Transmitter  fatal (Transmitter::Fatal);
Transmitter  warning (Transmitter::Warning);
TextReceiver text_receiver ("ARDOUR");

/* XXX remove us */

MIDI::MachineControl::CommandSignature MMC_CommandSignature;
MIDI::MachineControl::ResponseSignature MMC_ResponseSignature;

int 
setup_midi (MIDI::MachineControl **mmc)

{
	MIDI::Port *port;
	MIDI::PortRequest request (Config->midi_device.device,
				   Config->midi_device.tag,
				   Config->midi_device.mode,
				   Config->midi_device.type);
	
	if (request.status != MIDI::PortRequest::OK) {
		error << "MIDI port specifications are not understandable."
		      << endmsg;
		return -1;
	}

	port = MIDI::Manager::instance()->add_port (request);

	*mmc = 0;

	if (port == 0) {
		warning << "MIDI port \""
			<< Config->midi_device.tag
			<< "\" not available: no MMC or MTC control possible" 
			<< endmsg;
		return 0;
	} 

	(*mmc) = new MIDI::MachineControl (*port, 1.0, 
					   MMC_CommandSignature,
					   MMC_ResponseSignature);

	return 0;
}

int
setup_audio (ALSA::MultiChannelDevice **mcd)

{
	(*mcd) = new ALSA::MultiChannelDevice 
		(Config->audio_device.device,
		 Config->sample_rate, 
		 Config->frames_per_cycle,
		 Config->use_real_time);
	
	if (!(*mcd)->ok()) {
		error << "cannot create multi-channel device"
		      << endmsg;
		return -1;
	}

	return 0;
}

static ARDOUR_UI *ui;

static void
ardour_exit (void)

{
	if (ui) {
		ui->quit ();
	}
}

void
ardour_main (int argc, char *argv[])

{
	ALSA::MultiChannelDevice *mcd;
	MIDI::MachineControl *mmc;

	cout << "Ardour version " << ardour_version << endl
	     << "Copyright 2000  Paul Barton-Davis" << endl

#ifdef ARDOUR_WITH_SND
	     << " [ contains snd, by Bill Schottstaedt, " << endl
             << "   Copyright 1996-2000 The Board of Trustees of Stanford University ]" << endl
#endif ARDOUR_WITH_SND

	     << "Ardour comes with ABSOLUTELY NO WARRANTY" << endl
		
	     << "This is free software, and you are welcome to redistribute it "
	     << endl
	     << "under certain conditions; see the file COPYING for details"
	     << endl;

	text_receiver.listen_to (error);
	text_receiver.listen_to (info);
	text_receiver.listen_to (fatal);
	text_receiver.listen_to (warning);

	pthread_calibrate_spinlimit ();

	Config = new Configuration;

	if (Config->load_rcfile ()) {
		exit (1);
	}

	if (Config->use_real_time) {
		if (mlockall (MCL_CURRENT|MCL_FUTURE)) {
			warning << "cannot lock down all memory ("
				<< strerror (errno)
				<< ')'
				<< endmsg;
		}
	}

	if (setup_audio (&mcd)) {
		exit (1);
	}

	if (setup_midi (&mmc)) {
		exit (1);
	}

	ui = new ARDOUR_UI (mcd, mmc, &argc, &argv, 
			    Config->ui_rcfile.c_str());

	if (!ui->ok ()) {
		error << "could not create GUI" << endmsg;
		return;
	}

	atexit (ardour_exit);

	ui->run (text_receiver);

	/*NOTREACHED*/
	return;
}

int
main (int argc, char *argv[])

{
	gh_enter (argc, argv, ardour_main);
	return 0;
}
